#ifndef VECTORDEQUE_T_H
#define VECTORDEQUE_T_H

#include <algorithm>
#include <stdexcept>
#include <vector>

// Implement a deque in a vector, so that it can be accessed
// as one or two (when circular) old-style arrays.
template <class T>
class VectorDeque {
public:
	typedef typename std::vector<T>::size_type size_type;
private:
	std::vector<T> vec_;
	size_type first_, size_;
public:
	explicit VectorDeque(size_type capacity)
		: vec_(capacity), first_(0), size_(0) {}
	size_type Capacity() const
		{ return vec_.size(); }
	size_type Size() const
		{ return size_; }
	T & operator[](size_type i)
		{ return vec_[(first_ + i) % Capacity()]; }
	const T & operator[](size_type i) const
		{ return vec_[(first_ + i) % Capacity()]; }
	T * FirstRow()
		{ return &vec_[first_]; }
	const T * FirstRow() const
		{ return &vec_[first_]; }
	size_type FirstRowSize() const
		{ return std::min(Capacity() - first_, size_); }
	T * SecondRow()
		{ return &vec_[0]; }
	const T * SecondRow() const
		{ return &vec_[0]; }
	size_type SecondRowSize() const
		{ return size_ - FirstRowSize(); }
	void Push(T);
	void Pop();
	void Grow(size_type add_capacity);
};

template <class T>
void VectorDeque<T>::Push(T t)
{
	if (size_ >= Capacity()) throw std::runtime_error("Vectordeque overflow");
	vec_[(first_ + size_) % Capacity()] = t;
	++size_;
}

template <class T>
inline void VectorDeque<T>::Pop()
{
	first_ = (first_ + 1) % Capacity();
	--size_;
}

template <class T>
void VectorDeque<T>::Grow(size_type add_capacity)
{
	const size_type old_capacity (vec_.size());
	vec_.resize(old_capacity + add_capacity);
	if (first_ + size_ >= old_capacity) {
		std::copy(vec_.rbegin() + add_capacity,
		          vec_.rbegin() + (vec_.size() - first_),
			  vec_.rbegin());
		first_ += add_capacity;
	}
}

#endif
